'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global ClickWatcher, ElementChain, translate, changeTextDirection, getUnicodeDomain,
mayOpenDialogUI:true, relativeToAbsoluteUrl, BGcall  */

// Requires clickwatcher.js and elementchain.js and jQuery

// Create a selector that matches an element.
function selectorFromElm(el) {
  const attrs = ['id', 'class', 'name', 'src', 'href', 'data'];
  const result = [el.prop('nodeName')];
  for (let i = 0; i < attrs.length; i += 1) {
    const attr = attrs[i];
    const val = el.attr(attr);
    if (val) {
      result.push(`[${attr}=${JSON.stringify(val)}]`);
    }
  }

  return result.join('');
}

// Wizard that walks the user through selecting an element and choosing
// properties to block.
// clicked_item: the element that was right clicked, if any.
// advanced_user:bool
function BlacklistUi(clickedItem, advancedUser) {
  // If a dialog is ever closed without setting this to false, the
  // object fires a cancel event.
  this.cancelled = true;

  // steps through dialog - see _preview()
  this.current_step = 0;

  this.callbacks = { cancel: [], block: [] };

  this.clicked_item = clickedItem;
  this.advanced_user = advancedUser;
}

// TODO: same event framework as ClickWatcher
BlacklistUi.prototype.cancel = function cancelFunction(callback) {
  this.callbacks.cancel.push(callback);
};

BlacklistUi.prototype.block = function blockFunction(callback) {
  this.callbacks.block.push(callback);
};

BlacklistUi.prototype.fire = function fireFunction(eventName, arg) {
  const callbacks = this.callbacks[eventName];
  for (let i = 0; i < callbacks.length; i += 1) {
    callbacks[i](arg);
  }
};

BlacklistUi.prototype.onClose = function onCloseFunction() {
  if (this.cancelled === true) {
    this.ui_page1.empty().remove();
    this.ui_page2.empty().remove();
    $('.adblock-ui-stylesheet').remove();
    this.chain.current().show();
    this.fire('cancel');
  }
};

BlacklistUi.prototype.handle_change = function handleChangeFunction() {
  this.last.show();
  this.chain.current().hide();
  this.last = this.chain.current();
  this.redrawPage1();
  this.redrawPage2();
  this.preview(selectorFromElm(this.chain.current()));
};

BlacklistUi.prototype.show = function showFunction() {
  // If we don't know the clicked element, we must find it first.
  if (!this.clicked_item) {
    const clickWatcher = new ClickWatcher();
    const localThis = this;
    clickWatcher.cancel(() => {
      localThis.preview(null);
      localThis.fire('cancel');
    });

    clickWatcher.click((element) => {
      localThis.clicked_item = element;
      localThis.show();
    });

    this.preview('*');
    clickWatcher.show();
  } else {
    // If we do know the clicked element, go straight to the slider.
    this.chain = new ElementChain(this.clicked_item);

    this.ui_page1 = this.buildPage1();
    this.ui_page2 = this.build_page2();

    this.last = this.chain.current();
    this.chain.change(this, this.handle_change);
    this.chain.change();

    this.redrawPage1();
    this.ui_page1.dialog('open');
  }
};

BlacklistUi.prototype.buildPage1 = function buildPage1Function() {
  const localThis = this;

  const page = $('<div>')
    .text(translate('sliderexplanation'))
    .append('<br/>')
    .append("<input id='slider' type='range' min='0' value='0'/>")
    .append("<div id='selected_data'></div>");
  const btns = {};
  const adblockDefaultButtonText = translate('buttonlooksgood');
  btns[adblockDefaultButtonText] = {
    text: adblockDefaultButtonText,
    class: 'adblock_default_button',
    click(event) {
      event.stopPropagation();
      localThis.cancelled = false;
      localThis.ui_page1.dialog('close');
      localThis.cancelled = true;
      localThis.redrawPage2();
      localThis.ui_page2.dialog('open');
    },
  };
  btns[translate('buttoncancel')] =
      function closeDialog() {
        localThis.ui_page1.dialog('close');
      };

  page.dialog({
    classes: {
      'ui-dialog': 'adblock-blacklist-dialog',
    },
    closeText: '',
    position: { my: 'left top', at: 'left+10% top+10%', of: window },
    width: 410,
    autoOpen: false,
    title: translate('slidertitle'),
    buttons: btns,
    open(event) {
      event.stopPropagation();
      localThis.current_step = 1;
      localThis.preview(selectorFromElm(localThis.chain.current()));
    },

    close(event) {
      event.stopPropagation();
      localThis.preview(null);
      localThis.onClose();
    },
  });
  changeTextDirection($('body .adblock-blacklist-dialog'));

  let depth = 0;
  let guy = this.chain.current();
  while (guy.length > 0 && guy[0].nodeName !== 'BODY') {
    guy = guy.parent();
    depth += 1;
  }

  $('#slider', page)
    .attr('max', Math.max(depth - 1, 1))
    .on('input change', function moveChain() {
      localThis.chain.moveTo(this.valueAsNumber);
    });

  return page;
};

BlacklistUi.prototype.build_page2 = function buildPage2Function() {
  const localThis = this;
  const page = $('<div>').text(translate('blacklisteroptions1'))
    .append('<div>' +
          "<div id='adblock-details'></div><br/>" +
          "<div id='count'></div>" +
        '</div>')
    .append($('<div> <br/>')
      .text(translate('blacklisternotsure')))
    .append('<br/><br/>')
    .append($("<div style='clear:left; font-size:smaller; margin-top: -20px;'> <br/>")
      .text(translate('blacklisterthefilter')))
    .append("<div style='margin-left:15px;margin-bottom:15px'>" +
                '<div>' +
                  "<div id='summary'></div><br/>" +
                  "<div id='filter_warning'></div>" +
                '</div>' +
              '</div>' +
            '</div>' +
         '</div>');

  const btns = {};
  const adblockDefaultButtonText = translate('buttonblockit');
  btns[adblockDefaultButtonText] = {
    text: adblockDefaultButtonText,
    class: 'adblock_default_button',
    click(event) {
      event.stopPropagation();
      const rule = $('#summary', localThis.ui_page2).text();
      if (rule.length > 0) {
        const filter = `${getUnicodeDomain(document.location.hostname)}##${rule}`;

        // validateFilter will return the text passed to it, if the text is a valid filter
        // otherwise it a stringified exception is returned.
        BGcall('validateFilter', filter, true, (response) => {
          if (response === filter) {
            BGcall('addCustomFilter', filter, () => {
              mayOpenDialogUI = true;
              $('.adblock-ui-stylesheet').remove();
              page.remove();
              document.location.reload();
            });
          } else {
            try {
              const responseObj = JSON.parse(response);
              // eslint-disable-next-line no-alert
              alert(translate('customfilterserrormessage', [filter, responseObj.exception]));
              mayOpenDialogUI = true;
              $('.adblock-ui-stylesheet').remove();
              page.remove();
            } catch (ex) {
              // eslint-disable-next-line no-alert
              alert(translate('blacklisternofilter'));
              mayOpenDialogUI = true;
              $('.adblock-ui-stylesheet').remove();
              page.remove();
            }
          }
        });
      } else {
        // eslint-disable-next-line no-alert
        alert(translate('blacklisternofilter'));
      }
    },
  };
  if (localThis.advanced_user) {
    btns[translate('buttonedit')] =
      function editButtonFunction() {
        let customFilter = `${document.location.hostname}##${$('#summary', localThis.ui_page2).text()}`;
        localThis.ui_page2.dialog('close');
        // eslint-disable-next-line no-alert
        customFilter = prompt(translate('blacklistereditfilter'), customFilter);
        if (customFilter) { // null => user clicked cancel
          if (!/##/.test(customFilter)) {
            customFilter = `##${customFilter}`;
          }

          BGcall('validateFilter', customFilter, true, (response) => {
            if (response === customFilter) {
              BGcall('addCustomFilter', customFilter, () => {
                mayOpenDialogUI = true;
                $('.adblock-ui-stylesheet').remove();
                page.remove();
                document.location.reload();
              });
            } else {
              try {
                const responseObj = JSON.parse(response);
                // eslint-disable-next-line no-alert
                alert(translate('customfilterserrormessage', [customFilter, responseObj.exception]));
                mayOpenDialogUI = true;
                $('.adblock-ui-stylesheet').remove();
                page.remove();
              } catch (ex) {
                // eslint-disable-next-line no-alert
                alert(translate('blacklisternofilter'));
                mayOpenDialogUI = true;
                $('.adblock-ui-stylesheet').remove();
                page.remove();
              }
            }
          });
        }
      };
  }

  btns[translate('buttonback')] =
      function buttonBackFunction() {
        localThis.cancelled = false;
        localThis.ui_page2.dialog('close');
        localThis.cancelled = true;
        localThis.redrawPage1();
        localThis.ui_page1.dialog('open');
      };

  btns[translate('buttoncancel')] =
      function buttonCancelFunction() {
        localThis.ui_page2.dialog('close');
      };

  page.dialog({
    classes: {
      'ui-dialog': 'adblock-blacklist-dialog ui-page-2',
    },
    closeText: '',
    position: { my: 'left top', at: 'left+10% top+10%', of: window },
    width: 500,
    autoOpen: false,
    title: translate('blacklisteroptionstitle'),
    buttons: btns,
    open() {
      localThis.current_step = 2;
      localThis.preview($('#summary', localThis.ui_page2).text());
    },

    close() {
      localThis.preview(null);
      localThis.onClose();
    },
  });
  changeTextDirection($('body .adblock-blacklist-dialog'));

  return page;
};

BlacklistUi.prototype.redrawPage1 = function redrawPage1Function() {
  const el = this.chain.current();
  const showLink = (this.advanced_user &&
    /^https?:\/\//.test(relativeToAbsoluteUrl(el.attr('src') || el.attr('data'))));
  if (showLink) {
    $('#block_by_url_link').css('visibility', 'visible');
  }

  const $selectedData = $('#selected_data', this.ui_page1);
  $selectedData.append('<b>').text(translate('blacklisterblockedelement')).append('<br/>');

  $selectedData.append($('<i></i>').text(`<${el[0].nodeName}`));
  const attrs = ['id', 'class', 'name', 'src', 'href', 'data'];
  for (const i in attrs) {
    const val = BlacklistUi.ellipsis(el.attr(attrs[i]));
    if (val) {
      $selectedData.append('<br/>')
        .append($('<i></i>')
          .text(`${attrs[i]}="${val}"`)
          .css('margin-left', '10px'));
    }
  }

  $selectedData.append('<i>&nbsp;&gt;</i>');
};

// Return the CSS selector generated by the blacklister.  If the
// user has not yet gotten far enough through the wizard to
// determine the selector, return an empty string.
BlacklistUi.prototype.makeFilter = function makeFilerFunction() {
  const result = [];

  const el = this.chain.current();
  const $detailsDiv = $('#adblock-details', this.ui_page2);

  if ($("input[type='checkbox']#cknodeName", $detailsDiv).is(':checked')) {
    result.push(el.prop('nodeName'));

    // Some iframed ads are in a bland iframe.  If so, at least try to
    // be more specific by walking the chain from the body to the iframe
    // in the CSS selector.
    if (el.prop('nodeName') === 'IFRAME' && el.attr('id') === '') {
      let cur = el.parent();
      while (cur.prop('nodeName') !== 'BODY') {
        result.unshift(`${cur.prop('nodeName')} `);
        cur = cur.parent();
      }
    }
  }

  const attrs = ['id', 'class', 'name', 'src', 'href', 'data'];
  for (const i in attrs) {
    if ($(`input[type='checkbox']#ck${attrs[i]}`, $detailsDiv).is(':checked')) {
      result.push(`[${attrs[i]}=${JSON.stringify(el.attr(attrs[i]))}]`);
    }
  }

  let warningMessage;
  if (result.length === 0) {
    warningMessage = translate('blacklisterwarningnofilter');
  } else if (result.length === 1 && $("input[type='checkbox']#cknodeName", $detailsDiv).is(':checked')) {
    warningMessage = translate('blacklisterblocksalloftype', [result[0]]);
  }
  $('#filter_warning', this.ui_page2)
    .css('display', (warningMessage ? 'block' : 'none'))
    .text(warningMessage);
  return result.join('');
};

BlacklistUi.prototype.redrawPage2 = function redrawPage2Function() {
  const el = this.chain.current();
  const localThis = this;

  const $detailsDiv = $('#adblock-details', localThis.ui_page2);

  const $summary = $('#summary', localThis.ui_page2);

  function updateFilter() {
    const theFilter = localThis.makeFilter();

    $summary.text(theFilter);

    const matchCount = $(theFilter).not('.ui-dialog').not('.ui-dialog *').length;
    $('#count', localThis.ui_page2)
      .append('<center>')
      .text(() => {
        if (matchCount === 1) {
          return translate('blacklistersinglematch');
        }
        return translate('blacklistermatches', [matchCount]);
      });
  }

  $detailsDiv.empty();
  const attrs = ['nodeName', 'id', 'class', 'name', 'src', 'href', 'data'];
  for (let i = 0; i < attrs.length; i += 1) {
    const attr = attrs[i];
    const longVal = (attr === 'nodeName' ? el.prop('nodeName') : el.attr(attr));
    const val = BlacklistUi.ellipsis(longVal);

    if (val) {
      // Check src, data and href only by default if no other identifiers are
      // present except for the nodeName selector.
      let checked = true;
      if (attr === 'src' || attr === 'href' || attr === 'data') {
        checked = $('input', $detailsDiv).length === 1;
      }
      const checkboxlabel = $('<label></label>')
        .text(translate(
          'blacklisterattrwillbe',
          [(attr === 'nodeName' ? translate('blacklistertype') : attr), val],
        ))
        .attr('for', `ck${attr}`);

      const checkbox = $('<div></div>')
        .append(`<input type=checkbox ${checked ? 'checked="checked"' : ''
        } id=ck${attr} /> `)
        .append(checkboxlabel);

      // eslint-disable-next-line no-loop-func
      checkbox.find('input').change(() => {
        updateFilter();
        localThis.preview($('#summary', localThis.ui_page2).text());
      });

      $detailsDiv.append(checkbox);
    }
  }

  updateFilter();
};

// Change the appearance of a CSS selector on the page, or if null, undo the change.
// Inputs: selector:string - the selector generated by the blacklist wizard
BlacklistUi.prototype.preview = function previewFunction(selector) {
  $('#adblock_blacklist_preview_css').remove();
  if (!selector) {
    return;
  }

  const cssPreview = document.createElement('style');
  cssPreview.type = 'text/css';
  cssPreview.id = 'adblock_blacklist_preview_css';

  const d = 'body .adblock-blacklist-dialog';

  switch (this.current_step) {
    case 0:

    // Raise highlight.
      cssPreview.innerText = 'body .adblock-highlight-node,';
      break;
    case 1:

    // Show ui_page1.
      cssPreview.innerText = `${d}, ${d} * {opacity:1!important;} `;

      // Fade the selector, while skipping any matching children.
      cssPreview.innerText += `${selector} {opacity:.1!important;} ${
        selector} ${selector} {opacity:1!important;}`;
      break;
    case 2:

    // Show ui_page2.
      cssPreview.innerText = `${d} input, ${d
      } button {display:inline-block!important;} ${d}.ui-page-2, ${d
      } div:not(#filter_warning), ${d} .ui-icon, ${d} a, ${d
      } center {display:block!important;} ${d} #adblock-details, ${d
      } span, ${d} b, ${d} i {display:inline!important;} `;

      // Hide the specified selector.
      cssPreview.innerText += `${selector} {display:none!important;}`;
      break;
    default:
      break;
  }

  // Finally, raise the UI above *all* website UI, using max 32-bit signed int.
  cssPreview.innerText += ` ${d} {z-index:2147483647!important;}`;

  document.documentElement.appendChild(cssPreview);
};

// Return a copy of value that has been truncated with an ellipsis in
// the middle if it is too long.
// Inputs: value:string - value to truncate
//         size?:int - max size above which to truncate, defaults to 50
BlacklistUi.ellipsis = function ellipsisFunction(value, size) {
  if (!value) {
    return value;
  }

  let newSize = size;
  if (!newSize) {
    newSize = 50;
  }

  const half = (newSize / 2) - 2; // With ellipsis, the total length will be ~= newSize

  let newValue = value;
  if (newValue.length > newSize) {
    newValue = (`${newValue.substring(0, half)}...${
      newValue.substring(newValue.length - half)}`);
  }

  return newValue;
};

// required return value for tabs.executeScript
/* eslint-disable-next-line no-unused-expressions */
'';

// # sourceURL=/uiscripts/blacklisting/blacklistui.js
